@@ -4,7 +4,7 @@
 """ Reads in mzml file using pymzml and get list of ms2 scans """
 
 import logging
-
+import os
 import pymzml as mz
 
 
@@ -36,13 +36,14 @@ class Mzml(object):
         :return:
         """
 
-        #
         run = mz.run.Reader(self.path,
                             MS_precision={
                                 1: self.precision*1e-6,
                                 2: self.precision*1e-6
                             })
 
+
+
         for n, spec in enumerate(run):
 
             #if n % 1000 == 0:
