@@ -14,6 +14,7 @@ from pytmt import __version__
 from pytmt.get_spec import Mzml
 from pytmt import tmt_reporters
 from pytmt import quantify_spec
+from pytmt import correct_matrix
 
 def quant(args):
     """
@@ -82,7 +83,7 @@ def quant(args):
     mzml_loc = args.mzml
     id_loc = args.id
 
-    assert args.multiplex in [0, 2, 6, 10, 11, 16], '[error] TMT multiplexity not 0, 2, 6, 10, 11 or 16'
+    assert args.multiplex in [0, 2, 6, 10, 11, 16, 18], '[error] TMT multiplexity not 0, 2, 6, 10, 11, 16, or 18'
 
     # Get the reporter masses
     reporters = tmt_reporters.get_reporters(args.multiplex)
@@ -145,29 +146,64 @@ def quant(args):
     # Get all the file indices in the Percolator results file.
     file_indices = list(set(id_df['file_idx']))
 
-    # Check that the number of mzMLs in the mzML folder is the same as the maximum of the ID file's file_idx column.
-    # Note this will throw an error if not every fraction results in at least some ID, but we will ignore for now.
+    # 2022-03-28 pytmt will now attempt to read the percolator.log.txt file for fraction (file_idx) mzML assignment
+    log_path = os.path.join(os.path.dirname(id_loc), 'percolator.log.txt')
 
-    mzml_files = [f for f in os.listdir(mzml_loc) if re.match('^.*.mzML', f)]
+    # If the log file exists, use it to read the assignment
+    if os.path.exists(log_path):
+        main_log.warning(f'Percolator log file exists at {log_path} and will be used for index assignment.')
 
-    # Sort the mzML files by names
-    # Note this may create a problem if the OS Percolator runs on has natural sorting (xxxx_2 before xxxx_10)
-    # But we will ignore for now
-    mzml_files.sort()
+        with open(log_path, 'r') as f:
+            lines = f.readlines()
 
-    # Throw an error if there is no mzML file in the mzml directory
-    assert len(mzml_files) != 0, '[error] no mzml files in the specified directory'
-    assert len(mzml_files) == max(file_indices) + 1, '[error] number of mzml files not matching id list'
+        mzml_files = {}
+        for line in lines:
+            pattern = re.findall('INFO: Assigning index ([0-9]*) to (.*)\.', line)
+            if len(pattern) == 1:
+                idx, pathname = pattern[0]
+                dirname, filename = os.path.split(pathname)
+                mzml_files[int(idx)] = re.sub('\.pep\.xml', '', filename)
+                # TODO: will probably have to account for .pin or other input to Percolator
+
+    # If the log file does not exist, assign index naively based on sort
+    else:
+        main_log.warning(f'Percolator log file not found at {log_path}; '
+                         f'mzml files will be sorted for index assignment.')
+        # Check that the number of mzMLs in the mzML folder is the same as the maximum of the ID file's file_idx column.
+        # Note this will throw an error if not every fraction results in at least some ID, but we will ignore for now.
+
+        mzml_filelist = [f for f in os.listdir(mzml_loc) if re.match('^.*.mzML', f)]
+        # Sort the mzML files by names
+        # Note this may create a problem if the OS Percolator runs on has natural sorting (xxxx_2 before xxxx_10)
+        # But we will ignore for now
+        mzml_filelist.sort()
+
+        # Make dictionary of idx, filename from enumerate
+        mzml_files = {}
+        for idx, filename in enumerate(mzml_filelist):
+            mzml_files[idx] = re.sub('.mz[Mm][Ll](\.gz)?', '', filename)
+
+        # Throw an error if there is no mzML file in the mzml directory
+        assert len(mzml_files) != 0, '[error] no mzml files in the specified directory'
+        assert len(mzml_files) == max(file_indices) + 1, '[error] number of mzml files not matching id list'
+
+    # Print mzml files to log
+    main_log.info(f'mzml file orders: {mzml_files}')
 
     # For each file index (fraction), open the mzML file, and create a subset Percolator ID dataframe
     for idx in file_indices:
 
+        # 2022-03-28 try to open either mzML or mzML.gz
+        # TODO: may have to account for .mzml and mzml.gz rather than .mzML
+        if os.path.exists(os.path.join(mzml_loc, mzml_files[idx] + '.mzML')):
+            mzml_path = os.path.join(mzml_loc, mzml_files[idx] + '.mzML')
+        elif os.path.exists(os.path.join(mzml_loc, mzml_files[idx] + '.mzML.gz')):
+            mzml_path = os.path.join(mzml_loc, mzml_files[idx] + '.mzML.gz')
+        else:
+            raise FileNotFoundError
+
         # Logging mzML
-        main_log.info('Reading mzml file: {0} ({1} of {2})'.format(mzml_files[idx],
-                                                                   str(idx + 1),
-                                                                   str(len(file_indices)),
-                                                                   )
-                      )
+        main_log.info(f'Reading mzml file: {os.path.basename(mzml_path)} ({idx + 1} of {len(file_indices)})')
 
         # Make a subset dataframe with the current file index (fraction) being considered
         fraction_id_df = id_df[id_df['file_idx'] == idx]
@@ -176,7 +212,7 @@ def quant(args):
         fraction_id_df = fraction_id_df.sort_values(by='scan').reset_index(drop=True)
 
         # Open the mzML file
-        fraction_mzml = Mzml(path=os.path.join(mzml_loc, mzml_files[idx]),
+        fraction_mzml = Mzml(path=mzml_path,
                              precision=precision)
         fraction_mzml.parse_mzml_ms2()
 
@@ -200,7 +236,7 @@ def quant(args):
 
                 # Tidy this up
                 if spectrum is None:
-                    main_log.error('[error] spectrum index out of bound or is empty')
+                    main_log.error(f'[error] spectrum index {scan} out of bound or is empty')
                     raise KeyError
 
             # TODO: This doesn't give a KeyError currently. Need to catch when the id scans don't match the mzml.
@@ -244,10 +280,18 @@ def quant(args):
     for reporter in reporters:
         output_df_columns.append('m' + str(reporter))
 
-    output_df_columns.append('spectrum_int')
 
+
+    output_df_columns.append('spectrum_int')
     output_df = pd.DataFrame(output_list, columns=output_df_columns)
 
+    # Correct for contamination
+    if args.contam is not None:
+        output_df = correct_matrix.correct_matrix(output_df=output_df,
+                                                  contam=args.contam,
+                                                  nnls=args.nnls,
+                                                  )
+
     # Final output, merging the input and output tables
     final_df = pd.merge(id_df, output_df, how='left')
 
@@ -271,13 +315,22 @@ def main():
     import argparse
 
     parser = argparse.ArgumentParser(description='pytmt returns ms2 tmt quantification values'
-                                                 'from Percolator (Crux or Standalone) output')
+                                                 'from Percolator output and perform contamination'
+                                                 'correction')
 
-    parser.add_argument('mzml', help='path to folder containing mzml files')
+    parser.add_argument('mzml',
+                        help='path to folder containing mzml files',
+                        )
 
-    parser.add_argument('id', help='path to percolator target psms output file')
+    parser.add_argument('id',
+                        help='path to percolator target psms output file',
+                        )
 
-    parser.add_argument('-u', '--unique', action='store_true', help='quantify unique peptides only')
+    parser.add_argument('-u',
+                        '--unique',
+                        action='store_true',
+                        help='quantify unique peptides only',
+                        )
 
     parser.add_argument('-q', '--qvalue',
                         help='quantify peptides with q value below this threshold [default: 1.0]',
@@ -285,7 +338,7 @@ def main():
                         default=1.0)
 
     parser.add_argument('-m', '--multiplex',
-                        help='TMT-plex (0, 2, 6, 10, 11, 16) [default:10]',
+                        help='TMT-plex (0, 2, 6, 10, 11, 16, 18) [default:10]',
                         type=int,
                         default=10)
 
@@ -297,10 +350,21 @@ def main():
     parser.add_argument('-o', '--out', help='name of the output directory [default: tmt_out]',
                         default='tmt_out')
 
-
     parser.add_argument('-v', '--version', action='version',
                         version='%(prog)s {version}'.format(version=__version__))
 
+    parser.add_argument('-c', '--contam',
+                        help='Path to contaminant matrix csv file.'
+                             ' Leave blank to get tmt output without correction',
+                        metavar='CONTAM',
+                        type=argparse.FileType('r'),
+                        )
+
+    parser.add_argument('-n', '--nnls',
+                        action='store_true',
+                        help='uses non-negative least square for contamination correction',
+                        )
+
     parser.set_defaults(func=quant)
 
 
